c
c= FAKEOTF - Generate a fake RPFITS file from an existing one
c& tw
c: analysis
c+
c       FAKEOTF reads an RPFITS file and replaces all spectra with a
c       single Gaussian profile.  The amplitude of the Gaussian is
c       either constant across the map (fwhm=0) or falls of as a
c       circular 2-D Gaussian in the image plane.  A new RPFITS file is
c       produced.  Use this to simulate the effects of Gridzilla on
c       Mopra OTF data.
c       Limitation: The same parameters will be used for all IFs.
c
c@ in
c       Input RPFITS file, which should be an OTF map.  It is assumed
c       that the position and spectral timestamps match; if not run
c       MOPFIX first.
c@ out
c       Output RPFITS file.  No default.
c@ specpar
c       Amplitude and sigma of spectral Gaussian, in K and channel
c       units.  Default is 1,20 - i.e. 1 K and 20 channels.
c@ centre
c       RA,DEC of centre position for image Gaussian.  No default, but
c       can be omitted if fwhm=0.
c@ fwhm
c       FWHM of image Gaussian in arcsec (default 30). Give 0 for flat
c       plane.
c@ tsys
c       System temps for channels A and B respectively.  If only one
c       value is given, this is used for both channels.  This is also
c       used to determine the amount of noise in each spectrum.  Default
c       is to use the original Tsys measurements from the input file.
c
c$Id: fakeotf.for,v 1.1 2011/06/20 21:11:21 pteuben Exp $
c--
c 21jun05 - tw - created
c 20jul06 - tw - clean up for MOPS and miriad
c 16may07 - tw - detab; change label 900 from GOTO to CONTINUE
c-----------------------------------------------------------------------
      program fakeotf
c
c >>> ../../../src/inc/rpfits.inc
c--------------------- COMMONS FOR RPFITS ROUTINES ---------------------
c
c                  SEE RPFITS.DEFN FOR AN EXPLANATION
c
c     The FORTRAN standard requires that CHARACTER variables not be
c     mixed with other types in COMMON and hence they have a separate
c     COMMON block.  However, they are declared in context.
c
c     DOUBLE PRECISION variables were also segregated into a separate
c     COMMON block for reasons long forgotten.
c
c $Id: rpfits.inc,v 1.3 2011/06/01 05:17:17 pteuben Exp $
c-----------------------------------------------------------------------
c
c     Maximum numbers that we can handle: antenna table (ant), RPFITS
c     header cards, uncalibration table (cu), flag table (fg), IF table,
c     meteorological table (mt), index table (nx), syscal records (sc),
c     source table (su), and polarizations (pol).
      INTEGER   ant_max, max_card, max_cu, max_fg, max_if, max_mt,
     *          max_nx, max_sc, max_su, pol_max
      PARAMETER (ant_max  =   16,
     *           max_card = 2240,
     *           max_cu   =   32,
     *           max_fg   =   32,
     *           max_if   =   48,
     *           max_mt   =  256,
     *           max_nx   =  256,
     *           max_sc   =   16,
     *           max_su   = 2048,
     *           pol_max  =    8)
c
c
c     Error handling.
      INTEGER   errlun, rp_iostat
      CHARACTER errmsg*80
      COMMON /iostat/ rp_iostat, errlun
c
c     -------- Parameter values obtained from the RPFITS header --------
c                (except intbase which is a random parameter)
c
c     Miscellaneous parameters from RPFITS header cards (ncount, nfreq,
c     nstok, freq, dfreq, ra, dec, and coord are defunct; use if_nfreq,
c     if_nstok, if_freq, if_bw, if_nfreq su_ra, and su_dec, instead and
c     ignore ncount and coord - now always J2000).
      LOGICAL   write_wt
      INTEGER   data_format, intime, ncard, ncount, nfreq, nscan, nstok
      REAL      intbase
      DOUBLE PRECISION dec, dfreq, freq, ra
      CHARACTER bunit*16, cal*16, card(max_card)*80, coord*8, datobs*12,
     *          datsys*8, datwrit*12, file*256, instrument*16,
     *          object*16, obstype*16, rp_observer*16, rpfitsversion*20,
     *          version*20
      COMMON /param/ nstok, nfreq, ncount, intime, nscan, write_wt,
     *          ncard, intbase, data_format
c
c     Ephemeris parameters from RPFITS header cards.
      INTEGER   rp_defeat
      DOUBLE PRECISION rp_c(12), rp_djmrefp, rp_djmreft, rp_utcmtai
      COMMON /ephem/ rp_defeat
c
c     Proper motion parameters from RPFITS header cards.
      DOUBLE PRECISION pm_dec, pm_epoch, pm_ra
      COMMON /proper/ pm_ra, pm_dec, pm_epoch
c
c     Spectral line information from RPFITS header cards.
      INTEGER   ivelref
      DOUBLE PRECISION rfreq, vel1
      COMMON /spect/ ivelref
c
c     Antenna table (embedded in the RPFITS header).
      LOGICAL   an_found
      INTEGER   ant_mount(ant_max), ant_num(ant_max), nant
      DOUBLE PRECISION axis_offset(ant_max),
     *          feed_cal(ant_max,max_if,pol_max), feed_pa(2,ant_max),
     *          x(ant_max), x_array, y(ant_max), y_array, z(ant_max),
     *          z_array
      CHARACTER feed_type(2,ant_max)*2, sta(ant_max)*8
      COMMON /anten/ nant, ant_num, ant_mount, an_found
c
c     Uncalibration table (embedded in the RPFITS header).
      LOGICAL   cu_found
      INTEGER   cu_ant(max_cu), cu_ch1(max_cu), cu_ch2(max_cu),
     *          cu_if(max_cu), n_cu
      DOUBLE PRECISION cu_cal1(max_cu), cu_cal2(max_cu), cu_ut(max_cu)
      COMMON /cu/ n_cu, cu_ant, cu_if, cu_ch1, cu_ch2, cu_found
c
c     Flag table (embedded in the RPFITS header).
      LOGICAL   fg_found
      INTEGER   fg_ant(2,max_fg), fg_chan(2,max_fg),
     *          fg_if(2,max_fg), fg_stok(2,max_fg), n_fg
      DOUBLE PRECISION fg_ut(2,max_fg)
      CHARACTER fg_reason(max_fg)*24
      COMMON /fg/ n_fg, fg_ant, fg_if, fg_chan, fg_stok, fg_found
c
c     IF table (embedded in the RPFITS header).
      LOGICAL   if_found
      INTEGER   if_chain(max_if), if_invert(max_if), if_nfreq(max_if),
     *          if_nstok(max_if), if_num(max_if), if_sampl(max_if),
     *          if_simul(max_if), n_if
      DOUBLE PRECISION if_bw(max_if), if_freq(max_if), if_ref(max_if)
      CHARACTER if_cstok(4,max_if)*2
      COMMON /if/ n_if, if_invert, if_nfreq, if_nstok, if_sampl,
     *          if_found, if_num, if_simul, if_chain
c
c     Meteorological table (embedded in the RPFITS header).
      LOGICAL   mt_found
      INTEGER   mt_ant(max_mt), n_mt
      DOUBLE PRECISION mt_humid(max_mt), mt_press(max_mt),
     *          mt_temp(max_mt), mt_ut(max_mt)
      COMMON /mt/ n_mt, mt_ant, mt_found
c
c     Index table (embedded in the RPFITS header).
      LOGICAL   nx_found
      INTEGER   n_nx, nx_rec(max_nx)
      DOUBLE PRECISION nx_ut(max_nx)
      CHARACTER nx_date(max_nx)*12, nx_source(max_nx)*16
      COMMON /nx/ n_nx, nx_rec, nx_found
c
c     Source table (embedded in the RPFITS header).
      LOGICAL   su_found
      INTEGER   n_su, su_num(max_su)
      DOUBLE PRECISION su_dec(max_su), su_decd(max_su), su_pdec(max_su),
     *          su_pdecd(max_su), su_pra(max_su), su_prad(max_su),
     *          su_ra(max_su), su_rad(max_su)
      CHARACTER su_cal(max_su)*4, su_name(max_su)*16
      COMMON /su/ n_su, su_found, su_num
c
c
c     DOUBLE PRECISION variables (declared in context above).
      COMMON /doubles/ axis_offset, dec, dfreq, cu_cal1, cu_cal2, cu_ut,
     *          feed_cal, feed_pa, fg_ut, freq, if_bw, if_ref, if_freq,
     *          mt_humid, mt_press, mt_temp, mt_ut, nx_ut, ra, rfreq,
     *          rp_c, rp_djmrefp, rp_djmreft, rp_utcmtai, su_dec, su_ra,
     *          su_rad, su_decd, su_pra, su_pdec, su_prad, su_pdecd,
     *          vel1, x, x_array, y, y_array, z, z_array
c
c     CHARACTER variables (declared in context above).
      COMMON /names/ object, instrument, cal, rp_observer, datobs,
     *          datwrit, file, datsys, version, coord, sta, feed_type,
     *          card, if_cstok, su_name, su_cal, fg_reason, nx_source,
     *          nx_date, rpfitsversion, bunit, obstype, errmsg
c
c     -------------- Values obtained from the RPFITS data --------------
c
c     Syscal data.
      INTEGER   sc_ant, sc_if, sc_q, sc_srcno
      REAL      sc_cal(max_sc,max_if,ant_max), sc_ut
      COMMON /sc/ sc_ut, sc_ant, sc_if, sc_q, sc_cal, sc_srcno
c
c     ------------------------------------------------------------------
c
c     The following is for compatibility with early versions:
      DOUBLE PRECISION ant_humid(ant_max), ant_pressure(ant_max),
     *          ant_temp(ant_max), rp_humid(ant_max),
     *          rp_pressure(ant_max), rp_temp(ant_max)
      EQUIVALENCE (mt_humid, ant_humid,    rp_humid)
      EQUIVALENCE (mt_press, ant_pressure, rp_pressure)
      EQUIVALENCE (mt_temp,  ant_temp,     rp_temp)
c <<< ../../../src/inc/rpfits.inc
c
      double precision pi
      parameter (pi = 3.1415 92653 58979 3238 d0)
      integer MAXCHAN
      parameter (MAXCHAN=8192)
c
      character ctime*10, rastr*12, dcstr*12
      character infile*80, outfile*80, postab*20, src*16, versn*80
      integer jstat, flag, bin, if_no, source_no, baseline
      real ut, weight(2*MAXCHAN), u, v, w, sc_buffer(1)
      real width, amp0, amp, midch, func, imfwhm, noise,tsys(2),rms(2)
      double precision ractr, dcctr, imsig, dist2
      complex vis(2*MAXCHAN)
      integer az, el, tsysa, tsysb
      integer i, j, nsp, nhead, ncyc
      integer bw1, bw2, freq1, freq2, srclen
      logical isref, redotsys
      integer iptr, bufdim, iant, numif
      equivalence ( sc_buffer(1), sc_cal(1,1,1) )
c
c functions
      integer   len1
      character hangle*12, rangle*12, versan*80
c
c--------------------------------------------------------------
      versn = versan ('fakeotf',
     *                '$Revision: 1.1 $',
     *                '$Date: 2011/06/20 21:11:21 $')
c
c Get input parameters
      call keyini ()
      call keya ('in', infile, ' ')
      if (infile.eq.' ') then
         call bug('f','Input file must be given (in=)')
      endif
      call keya ('out', outfile, ' ')
      if (outfile.eq.' ') then
         call bug('f','Output file must be given (out=)')
      endif
c Centre for Gaussian source
      call keyt ('centre', ractr, 'hms', 0.d0)
      call keyt ('centre', dcctr, 'dms', 0.d0)
c FWHM of image Gaussian in arcsec
      call keyr ('fwhm', imfwhm, 30.)
c Width and peak amplitude of Gaussian line profile
      call keyr ('specpar', amp0, 1.)
      call keyr ('specpar', width, 20.)
c Width and peak amplitude of Gaussian line profile
      call keyr ('tsys', tsys(1), 0.)
      call keyr ('tsys', tsys(2), tsys(1))
      call keyfin ()
c
c-------------------------------------------------------
c
c Decide whether to redo Tsys
      redotsys = (tsys(1).gt.0)
c
c Convert imfwhm to sigma in radians
      imsig = imfwhm / (2.355*206265.)
c
c This card is needed for LiveData
      ncard = 1
      card(1) = 'OBSTYPE'
c
c Open the RPFITS file
c
      file = infile
      jstat = -3
      call rpfitsin (jstat, vis, weight, baseline, ut, u, v, w,
     *                        flag, bin, if_no, source_no)
c
      if (jstat .eq. -1) then
        write (6, '('' Error reading RPFITS file'')')
        goto 999
      end if
      write (6, '(a,a70)') ' Reading ', file
c
c Open the output RPFITS file
c
      file = outfile
      jstat = -3
      call rpfitsout (jstat, vis, weight, baseline, ut, u, v, w,
     *                        flag, bin, if_no, source_no)
      if (jstat .eq. -1) then
        write (6, '('' Error writing RPFITS file'')')
        goto 999
      end if
      write (6, '(a,a40)') ' Writing ', file
c
      nsp = 0
      nhead = 0
c
c BEGIN LOOP
      do while (.true.)
c
c Try to read data
c
        file = infile
        jstat = 0
        call rpfitsin (jstat, vis, weight, baseline, ut, u, v, w,
     *                 flag, bin, if_no, source_no)
c
c CASE 1: HEADER ENCOUNTERED OR END OF FILE
c
        if (jstat .eq. 1 .or. jstat .eq. 3) then
c
c Output the number of cycles in the preceding record
          if (nhead .gt. 0) then
             ncyc = nsp/n_if * nint(intbase/intime)
             if (ncyc .gt. 0) write(6,'(i3)') ncyc
          endif
          nhead = nhead + 1
c
c IF END OF FILE: close
c
          if (jstat .eq. 3) then
             write(6,*) 'END OF FILE'
             goto 999
          endif
c
c Flush the buffer of the output file
          if (nhead .gt. 0) then
             file = outfile
             jstat = 3
             call rpfitsout (jstat, vis, weight, baseline, ut,u,v,w,
     *                       flag, bin, if_no, source_no)
          endif
c
c load the header of the next scan
c
          file = infile
          jstat = -1
          call rpfitsin (jstat, vis, weight, baseline, ut, u, v, w,
     *                   flag, bin, if_no, source_no)
c
c and write it to the output file
          file = outfile
          jstat = -1
          call rpfitsout (jstat, vis, weight, baseline, ut, u, v, w,
     *                    flag, bin, if_no, source_no)
c
c report some parameters
c
          rastr = hangle(ra)
          dcstr = rangle(dec)
c
c -- figure out observation type
          src = su_name(1)
          srclen = len1(src)
          if (src(srclen-1:srclen).eq.'_R') then
             isref = .true.
          else
             isref = .false.
          endif
c
c get frequency info from header
          freq1 = nint(if_freq(1)/1e6)
          bw1 = nint(if_bw(1)/1e6)
          if (n_if .ge. 2) then
             freq2 = nint(if_freq(2)/1e6)
             bw2 = nint(if_bw(2)/1e6)
          else if (if_nstok(1) .eq. 2) then
             freq2 = freq1
             bw2 = bw1
          else
             freq2 = 0
             bw2 = 0
          endif
c
          nsp  = 0
          goto 900
c
        end if
c
c CASE 2: DATA IN SYSCAL RECORD
c
        if (baseline .eq. -1) then
           call format_time(sc_ut,ctime)
           iptr = 1
           bufdim = sc_q * sc_if * sc_ant
           do while (iptr .lt. bufdim)
              iant = (sc_buffer(iptr) + 0.5)
              if (iant .eq. 1) then
                 numif = nint(sc_buffer(iptr+1))
                 tsysa = nint(sc_buffer(iptr+3)**2)
                 tsysb = nint(sc_buffer(iptr+4)**2)
c Reset the Tsys values if requested
                 if (redotsys) then
                    sc_buffer(iptr+3) = sqrt(tsys(1))
                    sc_buffer(iptr+4) = sqrt(tsys(2))
                 else
                    tsys(1) = sc_buffer(iptr+3)**2
                    tsys(2) = sc_buffer(iptr+4)**2
                 endif
c Unflag records flagged by online system
                 if (nint(sc_buffer(iptr+12)).eq.1) then
                    sc_buffer(iptr+12) = 0.
                 endif
              else if (iant .eq. 0) then
                 az  = nint(sc_buffer(iptr+1)*180./pi)
                 el  = nint(sc_buffer(iptr+2)*180./pi)
              endif
              iptr = iptr + sc_q
           enddo
           if (numif.eq.1 .and. nsp.eq.0) then
              write(6,104) ctime,su_name(1),rastr,dcstr,az,el,
     *             freq1,freq2,bw1,bw2,tsysa,tsysb
           endif
 104       format(a8,1x,a10,1x,a10,1x,a9,i4,1x,i2,1x,i6,1x,i6,
     *            1x,i3,1x,i3,2i4,$)
c
           file = outfile
           jstat = 0
           call rpfitsout (jstat, vis, weight, baseline, ut, u, v, w,
     *                     flag, bin, if_no, source_no)
c
           goto 900
        endif
c
c CASE 3: CLOSE/REOPEN OPERATION
c
        if (jstat .eq. 5) then
           file = outfile
           jstat = 3
           call rpfitsout (jstat, vis, weight, baseline, ut, u, v, w,
     *                        flag, bin, if_no, source_no)
           goto 900
        endif
c
c CASE 4: DATA IN SPECTRUM
c
        nsp = nsp + 1
c
c Calculate model line profile
c
        if (.not. isref) then
           if (imfwhm .gt. 1e-10) then
              dist2 = ((ractr-u)*cos(dcctr))**2 + (dcctr-v)**2
              amp = amp0*exp(-dist2/(2.*imsig**2))
           else
              amp = amp0
           endif
           midch = if_nfreq(if_no)/2.
        endif
c
c Replace data with model
c
        do i = 1, if_nstok(if_no)
c          write(*,*) 'Pol ',i,' has ',if_nfreq(if_no),' chans'
           rms(i) = tsys(i)/sqrt(intime*if_bw(if_no)/if_nfreq(if_no))
c          write(*,*) 'The rms for pol ',i,' is ',rms(i)
           do j = 1, if_nfreq(if_no)
              call gaus(noise,1)
              if (.not. isref) then
                 func = (j-midch)**2/(2*width**2)
                 func = tsys(i) + amp*exp(-func) + noise*rms(i)
              else
                 func = tsys(i) + noise*rms(i)
              endif
              if (if_nstok(if_no) .eq. 2) then
                 vis(2*j-(2-i)) = cmplx(func)
              else
                 vis(j) = cmplx(func)
              endif
           enddo
        enddo
c
c Unset flags
        if (flag .ne. 0) then
           flag = 0
        endif
c
        file = outfile
        jstat = 0
        call rpfitsout (jstat, vis, weight, baseline, ut, u, v, w,
     *                  flag, bin, if_no, source_no)
c
 900  continue
      enddo
c
 999  continue
c
      file = infile
      jstat = 1
      call rpfitsin (jstat, vis, weight, baseline, ut, u, v, w,
     *               flag, bin, if_no, source_no)
      if (jstat.ne.0) THEN
         write(6,*)  'Error closing input file'
      endif
c
      file = outfile
      jstat = 1
      call rpfitsout (jstat, vis, weight, baseline, ut, u, v, w,
     *                flag, bin, if_no, source_no)
      if (jstat.ne.0) THEN
         write(6,*)  'Error closing output file'
      endif
c
      stop
 1000 write(*,*) 'Error opening ',postab
      stop
      end
c****************************************************
      subroutine  format_time (uts, tim_str)
c
      real ut,uts,left
      character*(*) tim_str
c
      integer ihr, imn, isec, fsec
c
c-------------------------------------------------
c
      ut = uts/3600.
      ihr = ut
      left = (ut - ihr) * 60.
      imn = left
      left = (left - imn) * 60.
      isec = left
      fsec = nint((left - isec)*10.)
c
      if (fsec .eq. 10) then
         isec = isec + 1
         fsec = 0
      endif
c
      if (isec .ge. 60) then
        isec = isec - 60
        imn = imn + 1
      end if
c
      if (imn .ge. 60) then
        imn = imn - 60
        ihr = ihr + 1
      end if
c
      write (tim_str, 10) ihr, imn, isec, fsec
 10   format (I2.2, ':', I2.2, ':', I2.2, '.', I1)
c
      return
      end
