c***********************************************************************
        program julian
c
c   This task computes the Julian Day given an input date. The
c   reverse calculation, Julian Day to a formatted date, is also
c   possible. With no arguments, JULIAN will convert the current
c   date and time to Julian Day.
c
c
c  History:
c      mwp  27-aug-1999 original version
c      pjt   2-dec-2005 added MJD
c      pjt  12-dec-2012 cleaned up and added options=date
c      pjt  15-oct-2013 jday becomes mjd if < 2400000
c
c------ Inline doc (retrieved with doc to a .doc file) --------------72]
c
c= julian - Julian Day conversion
c& mwp
c: misc
c+
c   JULIAN
c
c   This task computes the Julian Day given an input date. The
c   reverse calculation, Julian Day to a formatted date, is also
c   possible. With no arguments, JULIAN will convert the current
c   date and time to Julian Day.
c   A modified julian day is also printed out, which is simply
c   JD - 2400000.5
c   For reference. MJD2000 = 51544.0, JD2000 = 2451544.5
c
c
c@ date
c	The formatted date to convert. It must be one of the following forms:
c                     `yymmmdd.dd'                     (D)
c             or:
c                     `dd/mm/yy'                       (F)
c             or:
c                     `[yymmmdd:][hh[:mm[:ss.s]]]'     (H)
c             or:
c                     `ccyy-mm-dd[Thh[:mm[:ss.s]]]'    (T)
c             or:
c                     `dd-mmm-ccyy'                    (V)
c
c	Leave this input unspecified if you are converting from Julian Day
c       to date.
c
c
c@ jday
c	The Julian Day in floating point format. If given, the program
c       will convert it to a date, formatted by FORMAT keyword specification.
c	Leave this input unspecified if you are converting from date
c       to Julian Day.
c       If the value is less than 2400000.5, it is assumed MJD is meant
c       and 2400000.5 is added to get the true JD.
c
c@ format
c 	The requested OUTPUT format for converting a Julian day to a
c	date. It is indicated by a single letter [DFHTV], which corresponds
c       to the date formats given above [default=H]. This input is ignored for
c 	date to Julian day conversions
c
c@ options
c       date  - do conversion of dates, no display of julian
c
c	quiet - report only the answer and not other info. Useful
c		if you are using this program in a script.
c
c
c-----------------------------------------------------------------------
        character version*(*)
        double precision jday0
c
c the string date to convert
        character*32 thedate
c the output format, one of DFHTV
        character theform
c the julian day
        double precision jday
c which way are we going? forward=true means date-->julian day
        logical forward
c verbose output format or not
        logical quiet,qdate
c
c referenced routines
        integer len1
        logical islowerf
c
        parameter(version='(Version 15-oct-2013)')
        parameter(jday0=2400000.5d0)
c
        call keyini
        call keyd('jday',jday,-999d0)
        call keya('date',thedate,' ')
        call keya('format',theform,'H')
        call GetOpt(quiet,qdate)
        call keyfin
c
        if(.not.quiet) call output('JULIAN: '//version)
c-----------------------------------------------------------------------
c IF date == ' ' && jday == -999, call todayjul
c IF date != ' ' call dayjul
c IF jdat != -999 call julday
c-----------------------------------------------------------------------
c
c
c check inputs
c
        if(islowerf(theform)) call ucase(theform)
c
        if (  (theform.ne.'D').and.(theform.ne.'F')
     *   .and.(theform.ne.'H').and.(theform.ne.'T')
     *   .and.(theform.ne.'V')) then
                call bug('f','format= must be one of D,F,H,T,V')
        endif
c
        if (jday.lt.0d0) then
           forward=.true.
           if (thedate.eq.' ') then
              call todayjul(jday)
           else
              call dayjul(thedate,jday)
           endif
           if (qdate) then
              call julday(jday,theform,thedate)
              forward = .false.
           endif
        else
           if (jday.lt.2000000d0) jday = jday + jday0
           if (thedate.ne.' ') then
              call bug('w',
     *           'Specify only one of date= or jday=. Ignoring date.')
           endif
           forward=.false.
           call julday(jday,theform,thedate)
        endif
c
        if(forward) then
           if(thedate.eq.' ') call julday(jday,theform,thedate)
           if(.not.quiet) then
              write(6,1001) thedate(1:len1(thedate)), jday
              write(6,1005) thedate(1:len1(thedate)), jday-jday0
           else
              write(6,1003) jday
           endif
        else
           if(.not.quiet) then
              write(6,1002) jday,thedate(1:len1(thedate))
           else
              write(6,1004) thedate(1:len1(thedate))
           endif
        endif
c
 1001   format(a,' is Julian Day ',f15.6)
 1002   format('Julian Day ',f15.6,' is the date ',a)
 1003   format(f15.6)
 1004   format(a)
 1005   format(a,' is Modified Julian Day ',f14.6)
c
c
        end
c********1*********2*********3*********4*********5*********6*********7**
        subroutine GetOpt(quiet,date)
c
        logical quiet,date
c
c  Get extra processing options.
c------------------------------------------------------------------------
        integer nopts
        parameter(nopts=2)
        logical present(nopts)
        character opts(nopts)*8
        data opts/'quiet ','date '/
c
        call options('options',opts,present,nopts)
        quiet = present(1)
        date  = present(2)
        end
