c***********************************************************************
c  A set of routines for formatting angles and times.
c
c  History:
c    Refer to the RCS log, v1.1 includes prior revision information.
c
c $Id: angles.for,v 1.2 2012/05/11 04:46:29 pteuben Exp $
c***********************************************************************
c* dangle - Convert degrees/hours value into a formatted string.
c& jm
c: utilities
c+
      character*(*) function dangle(theta)
c
      double precision theta
c  ---------------------------------------------------------------------
c  Convert an angle expressed in degrees or hours into a string.
c
c  Inputs:
c    theta    Angle in decimal degrees or hours.
c
c  Output:
c    dangle   Angle formated into a string with format:
c               [+/-]DD:MM:SS.SS
c-----------------------------------------------------------------------
      double precision ROUND
      parameter (ROUND=0.5d0/3600d2)
c
      double precision deg
      character text*12
c-----------------------------------------------------------------------
      deg = abs(theta) + ROUND
      write(text,10) int(deg), mod(int(60d0*deg),60),
     *  mod(int(3600d0*deg),60), mod(int(3600d2*deg),100)
 10   format(i3.2,':',i2.2,':',i2.2,'.',i2.2)
c
      if (theta.lt.0d0) then
        dangle = '-'
      else
        dangle = '+'
      endif
c
      if (text(1:1).ne.' ') then
        dangle(2:) = text
      else if (text(2:2).ne.' ') then
        dangle(2:) = text(2:)
      else
        dangle(2:) = text(3:)
      endif
c
      end
c***********************************************************************
c* dangleh - Convert degrees/hours value into a formatted string
c& jm
c: utilities
c+
      character*(*) function dangleh(theta)
c
      double precision theta
c  ---------------------------------------------------------------------
c  Convert an angle expressed in degrees or hours into a string.
c  Gives one more decimal place than dangle in the seconds string.
c
c  Inputs:
c    theta    Angle in decimal degrees or hours.
c
c  Output:
c    dangleh  Angle formated into a string with format:
c               [+/-]DD:MM:SS.SSS
c-----------------------------------------------------------------------
      double precision ROUND
      parameter (ROUND=0.5d0/3600d3)
c
      double precision deg
      character text*13
c-----------------------------------------------------------------------
      deg = abs(theta) + ROUND
      write(text,10) int(deg), mod(int(60d0*deg),60),
     *  mod(int(3600d0*deg),60), mod(int(3600d3*deg),1000)
 10   format(i3.2,':',i2.2,':',i2.2,'.',i3.3)
c
      if (theta.lt.0d0) then
        dangleh = '-'
      else
        dangleh = '+'
      endif
c
      if (text(1:1).ne.' ') then
        dangleh(2:) = text
      else if (text(2:2).ne.' ') then
        dangleh(2:) = text(2:)
      else
        dangleh(2:) = text(3:)
      endif
c
      end
c***********************************************************************
c* rangle - Convert degrees value (in radians) into a formatted string.
c& jm
c: utilities
c+
      character*(*) function rangle(theta)
c
      double precision theta
c  ---------------------------------------------------------------------
c  Convert an angle expressed in radians into a string.
c
c  Inputs:
c    theta    Angle in radians.
c
c  Output:
c    rangle   Angle formated into a string with format:
c               [+/-]DD:MM:SS.SS
c-----------------------------------------------------------------------
c >>> ../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../src/inc/mirconst.h
c
      external  dangle
      character dangle*13
c-----------------------------------------------------------------------
      rangle = dangle(theta*DR2D)
c
      end
c***********************************************************************
c* rangleh - Convert degrees value (in radians) into a formatted string.
c& jm
c: utilities
c+
      character*(*) function rangleh(theta)
c
      double precision theta
c  ---------------------------------------------------------------------
c  Convert an angle expressed in radians into a string.  Gives one more
c  decimal place than rangle in the seconds string.
c
c  Inputs:
c    theta    Angle in radians.
c
c  Output:
c    rangleh  Angle formated into a string with format:
c               [+/-]DD:MM:SS.SSS
c-----------------------------------------------------------------------
c >>> ../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../src/inc/mirconst.h
c
      external  dangleh
      character dangleh*14
c-----------------------------------------------------------------------
      rangleh = dangleh(theta*DR2D)
c
      end
c***********************************************************************
c* hangle - Convert hours value (in radians) into a formatted string.
c& jm
c: utilities
c+
      character*(*) function hangle(theta)
c
      double precision theta
c  ---------------------------------------------------------------------
c  Convert an angle expressed in radians into a string.
c
c  Inputs:
c    theta    Angle in radians.
c
c  Output:
c    rangle   Angle formated into a string with format:
c               [+/-]HH:MM:SS.SS
c-----------------------------------------------------------------------
c >>> ../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../src/inc/mirconst.h
c
      character text*13
c
      external  dangle
      character dangle*13
c-----------------------------------------------------------------------
      text = dangle(theta*DR2D/15d0)
c
      if (text(:1).eq.'+') then
        hangle = text(2:)
      else
        hangle = text
      endif
c
      end
c***********************************************************************
c* hangleh - Convert hours value (in radians) into a formatted string.
c& jm
c: utilities
c+
      character*(*) function hangleh(theta)
c
      double precision theta
c  ---------------------------------------------------------------------
c  Convert an angle expressed in radians into a string.  Gives one more
c  decimal place than hangle in the seconds string.
c
c  Inputs:
c    theta    Angle in radians.
c
c  Output:
c    rangle   Angle formated into a string with format:
c               [+/-]HH:MM:SS.SSS
c-----------------------------------------------------------------------
c >>> ../../src/inc/mirconst.h
c=======================================================================
c - mirconst.h  Include file for various fundamental physical constants.
c
c  History:
c    jm  18dec90  Original code.  Constants taken from the paper
c                 "The Fundamental Physical Constants" by E. Richard
c                 Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
c   pjt  13aug08  Aligned with ATNF
c   pjt  14nov08  updated from latest NIST values
c                 http://physics.nist.gov/cuu/Constants/index.html
c
c $Id: mirconst.h,v 1.3 2008/11/15 04:22:29 pteuben Exp $
c ----------------------------------------------------------------------
c  Pi.
      double precision DPI, DPI_2, DPI_4, DTWOPI
      parameter (DPI = 3.14159265358979323846D0)
      parameter (DTWOPI = DPI * 2D0)
      parameter (DPI_2  = DPI / 2D0)
      parameter (DPI_4  = DPI / 4D0)
c
      real PI, PI_2, PI_4, TWOPI
      parameter (PI    = DPI)
      parameter (TWOPI = DTWOPI)
      parameter (PI_2  = DPI_2)
      parameter (PI_4  = DPI_4)
c ----------------------------------------------------------------------
c  Angular conversion factors.
      double precision DAS2R, DD2R, DR2AS, DR2D
      parameter (DD2R  =  DPI / 180D0)
      parameter (DR2D  =  1D0 / DD2R)
      parameter (DAS2R = DD2R / 3600D0)
      parameter (DR2AS =  1D0 / DAS2R)
c
      real AS2R, D2R, R2AS, R2D
      parameter (D2R   = DD2R)
      parameter (R2D   = DR2D)
      parameter (AS2R  = DAS2R)
      parameter (R2AS  = DR2AS)
c ----------------------------------------------------------------------
c  Speed of light (meter/second).
      double precision DCMKS
      parameter (DCMKS = 299792458D0)
c
      real CMKS
      parameter (CMKS = DCMKS)
c ----------------------------------------------------------------------
c  Boltzmann constant (Joule/Kelvin).
      double precision DKMKS
      parameter (DKMKS = 1.3806504D-23)
c
      real KMKS
      parameter (KMKS = DKMKS)
c ----------------------------------------------------------------------
c  Planck constant (Joule-second).
      double precision DHMKS
      parameter (DHMKS = 6.62606896D-34)
c
      real HMKS
      parameter (HMKS = DHMKS)
c ----------------------------------------------------------------------
c  Planck constant divided by Boltzmann constant (Kelvin/GHz).
      double precision DHOVERK
      parameter (DHOVERK = 1D9 * DHMKS / DKMKS)
c
      real HOVERK
      parameter (HOVERK = DHOVERK)
c=======================================================================
c <<< ../../src/inc/mirconst.h
c
      character text*13
c
      external  dangle
      character dangleh*13
c-----------------------------------------------------------------------
      text = dangleh(theta*DR2D/15d0)
c
      if (text(:1).eq.'+') then
        hangleh = text(2:)
      else
        hangleh = text
      endif
c
      end
