c* NUMBPG -- convert a number into a plottable character string
c& MJS
c: STRINGS
c+
      SUBROUTINE NUMBPG (MM, PP, FORM, STRING, NC)
      INTEGER MM, PP, FORM
      CHARACTER*(*) STRING
      INTEGER NC
c
c This routine converts a number into a decimal character
c representation. To avoid problems of floating-point roundoff, the
c number must be provided as an integer (MM) multiplied by a power of 10
c (10**PP).  The output string retains only significant digits of MM,
c and will be in either integer format (123), decimal format (0.0123),
c or exponential format (1.23x10**5). Standard escape sequences \u, \d
c raise the exponent and \x is used for the multiplication sign.
c This routine is used by PGBOX to create numeric labels for a plot.
c
c Formatting rules:
c   (a) Decimal notation (FORM=1):
c       - Trailing zeros to the right of the decimal sign are
c         omitted
c       - The decimal sign is omitted if there are no digits
c         to the right of it
c       - When the decimal sign is placed before the first digit
c         of the number, a zero is placed before the decimal sign
c       - The decimal sign is a period (.)
c       - No spaces are placed between digits (ie digits are not
c         grouped in threes as they should be)
c       - A leading minus (-) is added if the number is negative
c   (b) Exponential notation (FORM=2):
c       - The exponent is adjusted to put just one (non-zero)
c         digit before the decimal sign
c       - The mantissa is formatted as in (a), unless its value is
c         1 in which case it and the multiplication sign are omitted
c       - If the power of 10 is not zero and the mantissa is not
c         zero, an exponent of the form \x10\u[-]nnn is appended,
c         where \x is a multiplication sign (cross), \u is an escape
c         sequence to raise the exponent, and as many digits nnn
c         are used as needed
c   (c) Automatic choice (FORM=0):
c         Decimal notation is used if the absolute value of the
c         number is less than 10000 or greater than or equal to
c         0.01. Otherwise exponential notation is used.
c
c Arguments:
c  MM     (input)
c  PP     (input)  : the value to be formatted is MM*10**PP.
c  FORM   (input)  : controls how the number is formatted:
c                    FORM = 0 -- use either decimal or exponential
c                    FORM = 1 -- use decimal notation
c                    FORM = 2 -- use exponential notation
c  STRING (output) : the formatted character string, left justified.
c                    If the length of STRING is insufficient, a single
c                    asterisk is returned, and NC=1.
c  NC     (output) : the number of characters used in STRING:
c                    the string to be printed is STRING(1:NC).
c--
c  History:
c
c     mjs  29mar92 Taken as-is from PGPLOT's "pgnumb" to avoid the
c                  need to load all of PGPLOT when only that one
c                  routine is needed (relevent to shared library
c                  implementations).
c     mjs  14jun93 Incorporate TJP's mod to original "pgnumb".
c-----------------------------------------------------------------------
      CHARACTER*1 BSLASH
      CHARACTER*2 TIMES, UP, DOWN
      CHARACTER*20 WORK, WEXP, TEMP
      INTEGER M, P, ND, I, J, K, NBP
      LOGICAL MINUS
c
c Define backslash (escape) character and escape sequences.
c
      BSLASH = CHAR(92)
      TIMES  = BSLASH//'x'
      UP     = BSLASH//'u'
      DOWN   = BSLASH//'d'
c
c Zero is always printed as "0".
c
      IF (MM.EQ.0) THEN
          STRING = '0'
          NC = 1
          RETURN
      END IF
c
c If negative, make a note of that fact.
c
      MINUS = MM.LT.0
      M = ABS(MM)
      P = PP
c
c Convert M to a left-justified digit string in WORK. As M is a
c positive integer, it cannot use more than 10 digits (2147483647).
c
      J = 10
   10 IF (M.NE.0) THEN
          K = MOD(M,10)
          M = M/10
          WORK(J:J) = CHAR(ICHAR('0')+K)
          J = J-1
       GOTO 10
      END IF
      TEMP = WORK(J+1:)
      WORK = TEMP
      ND = 10-J
c
c Remove right-hand zeros, and increment P for each one removed.
c ND is the final number of significant digits in WORK, and P the
c power of 10 to be applied. Number of digits before decimal point
c is NBP.
c
   20 IF (WORK(ND:ND).EQ.'0') THEN
          ND = ND-1
          P = P+1
       GOTO 20
      END IF
      NBP = ND+MIN(P,0)
c
c Integral numbers of 4 or less digits are formatted as such.
c
      IF ((P.GE.0) .AND. (P+ND.LE.4)) THEN
          DO 30 I=1,P
              ND = ND+1
              WORK(ND:ND) = '0'
   30     CONTINUE
          P = 0
c
c If NBP is 4 or less, simply insert a decimal point in the right place.
c
      ELSE IF (NBP.GE.1.AND.NBP.LE.4.AND.NBP.LT.ND) THEN
          TEMP = WORK(NBP+1:ND)
          WORK(NBP+2:ND+1) = TEMP
          WORK(NBP+1:NBP+1) = '.'
          ND = ND+1
          P = 0
c
c Otherwise insert a decimal point after the first digit, and adjust P.
c
      ELSE
          P = P + ND - 1
          IF (P.EQ.-1) THEN
              TEMP = WORK
              WORK = '0'//TEMP
              ND = ND+1
              P = 0
          ELSE IF (P.EQ.-2) THEN
              TEMP = WORK
              WORK = '00'//TEMP
              ND = ND+2
              P = 0
          END IF
          IF (ND.GT.1) THEN
              TEMP = WORK(2:ND)
              WORK(3:ND+1) = TEMP
              WORK(2:2) = '.'
              ND = ND + 1
          END IF
      END IF
c
c Add exponent if necessary.
c
      IF (P.NE.0) THEN
          WORK(ND+1:ND+6) = TIMES//'10'//UP
          ND = ND+6
          IF (P.LT.0) THEN
              P = -P
              ND = ND+1
              WORK(ND:ND) = '-'
          END IF
          J = 10
   40     IF (P.NE.0) THEN
              K = MOD(P,10)
              P = P/10
              WEXP(J:J) = CHAR(ICHAR('0')+K)
              J = J-1
           GOTO 40
          END IF
          WORK(ND+1:) = WEXP(J+1:10)
          ND = ND+10-J
          IF (WORK(1:3).EQ.'1'//TIMES) THEN
              TEMP = WORK(4:)
              WORK = TEMP
              ND = ND-3
          END IF
          WORK(ND+1:ND+2) = DOWN
          ND = ND+2
      END IF
c
c Add minus sign if necessary and move result to output.
c
      IF (MINUS) THEN
         TEMP = WORK(1:ND)
         STRING = '-'//TEMP
         NC = ND+1
      ELSE
         STRING = WORK(1:ND)
         NC = ND
      END IF
c
c Check result fits.
c
      IF (NC.GT.LEN(STRING)) THEN
          STRING = '*'
          NC = 1
      END IF
      END
