c************************************************************************
c
c  A collection of routines to determine the velocity of a point on
c  the earth, with respect to various rest frames.
c
c  These have been stolen and adapted from the SLALIB Starlink routines
c  by P.T.Wallace (circa 1986).
c
c  History:
c    13jul93 rjs  Original Miriad version.
c     2nov01 rjs  Changed definition of LSR velocity by 0.3 km/s
c************************************************************************
      SUBROUTINE VEARTH (JDAY, POS, VEL)
c
      DOUBLE PRECISION JDAY,POS(3),VEL(3)
c
c  Approximate heliocentric position and velocity of the Earth
c  The date and time should really be in the TDB in the Gregorian
c  calendar, and is interpreted in a manner which is valid between
c  1900 March 1 and 2100 February 28.
c
c  Input:
c    jday	Time of interest (as Julian day).
c  Output:
c    pos	Position, in km.
c    vel	Velocity, in km/sec.
c
c  The Earth heliocentric position/velocity is for mean equator and equinox
c  of date.
c
c  Max/RMS errors 1950-2050:
c     13/5 E-5 AU = 19200/7600 km in position
c     47/26 E-10 AU/s = 0.0070/0.0039 km/s in speed
c------------------------------------------------------------------------
      REAL TWOPI,SPEED,REMB,SEMB
      DOUBLE PRECISION AUKM,J1901
      REAL YF,T,ELM,GAMMA,EM,ELT,EPS0,DAY,
     *     E,ESQ,V,R,ELMM,COSELT,SINEPS,COSEPS,W1,W2,SELMM,CELMM
      INTEGER IY,QUAD
c
      PARAMETER (TWOPI=6.28318530718)
c
c  Mean orbital speed of Earth, AU/s
      PARAMETER (SPEED=1.9913E-7)
c
c  Mean Earth:EMB distance and speed, AU and AU/s
      PARAMETER (REMB=3.12E-5,SEMB=8.31E-11)
c
c AU to km.
      PARAMETER(AUKM=149.597870D6)
c
c Julian date for 1 January, 1901.
      PARAMETER(J1901=2415385.5d0)
c
c  Whole years & fraction of year, and years since 1900.
      QUAD = INT(JDAY - J1901) / 1461
      IY   = INT(JDAY - J1901 - 1461*QUAD) / 365
      DAY   = JDAY - J1901 - 1461*QUAD - 365*IY + 1
c
      IY = 4*QUAD + IY + 1
      YF = (4*DAY - 4*(1/(MOD(IY,4)+1)) - MOD(IY,4) - 2) / 1461.0
      T = IY + YF
c
c  Geometric mean longitude of Sun
c  (cf 4.881627938+6.283319509911*T MOD 2PI)
      ELM=MOD(4.881628+TWOPI*YF+0.00013420*T,TWOPI)
c
c  Mean longitude of perihelion
      GAMMA=4.908230+3.0005E-4*T
c
c  Mean anomaly
      EM=ELM-GAMMA
c
c  Mean obliquity
      EPS0=0.40931975-2.27E-6*T
c
c  Eccentricity
      E=0.016751-4.2E-7*T
      ESQ=E*E
c
c  True anomaly
      V=EM+2.0*E*SIN(EM)+1.25*ESQ*SIN(2.0*EM)
c
c  True ecliptic longitude
      ELT=V+GAMMA
c
c  True distance
      R=(1.0-ESQ)/(1.0+E*COS(V))
c
c  Moon's mean longitude
      ELMM=MOD(4.72+83.9971*T,TWOPI)
c
c  Useful functions
      COSELT=COS(ELT)
      SINEPS=SIN(EPS0)
      COSEPS=COS(EPS0)
      W1=-R*SIN(ELT)
      W2=-SPEED*(COSELT+E*COS(GAMMA))
      SELMM=SIN(ELMM)
      CELMM=COS(ELMM)
c
c  Earth position and velocity
      POS(1) = AUKM * (-R*COSELT-REMB*CELMM)
      POS(2) = AUKM * (W1-REMB*SELMM)*COSEPS
      POS(3) = AUKM * W1*SINEPS
      VEL(1) = AUKM * (SPEED*(SIN(ELT)+E*SIN(GAMMA))+SEMB*SELMM)
      VEL(2) = AUKM * (W2-SEMB*CELMM)*COSEPS
      VEL(3) = AUKM * W2*SINEPS
c
      END
c***********************************************************************
      SUBROUTINE VSITE(PHI, ST, VEL)
      DOUBLE PRECISION PHI, ST, VEL(3)
c+
c
c  Velocity due to Earth rotation
c
c  Input:
c     PHI	latitude of observing station (geodetic)
c     ST 	local apparent sidereal time
c  Output:
c     VEL	velocity in km/s.
c
c  PHI and ST are all in radians.
c  Accuracy:
c     The simple algorithm used assumes a spherical Earth and
c     an observing station at sea level.  For actual observing
c     sites, the error is unlikely to be greater than 0.0005 km/s.
c-----------------------------------------------------------------------
c
c  Sidereal speed of Earth equator, adjusted to compensate for
c  the simple algorithm used.  (The true value is 0.4651.)
      REAL ESPEED
      PARAMETER (ESPEED=0.4655)
c
      VEL(1) = -ESPEED*COS(PHI)*SIN(ST)
      VEL(2) =  ESPEED*COS(PHI)*COS(ST)
      VEL(3) = 0d0
      END
c***********************************************************************
      SUBROUTINE VSUN(VEL)
      DOUBLE PRECISION VEL(3)
c
c  Velocity of the Sun with respect to the Local Standard of Rest
c
c  Output:
c     VEL	Velocity of the Sun.
c------------------------------------------------------------------------
c
c  Speed = 20 km/s
c
c  Apex = RA 270 deg, Dec +30deg, 1900.0
c  = 18 07 50.3, +30 00 52, J2000.0
c
c  This is expressed in the form of a J2000.0 x,y,z vector:
c
c      VA(1) = X = -SPEED*COS(RA)*COS(DEC)
c      VA(2) = Y = -SPEED*SIN(RA)*COS(DEC)
c      VA(3) = Z = -SPEED*SIN(DEC)
      REAL VA(3)
      DATA VA / -0.29000, +17.31726, -10.00141 /
      VEL(1) = VA(1)
      VEL(2) = VA(2)
      VEL(3) = VA(3)
      END
